from .drone_deploy import DroneDeploy
from .aerialcactus import AerialCactus
from .resisc45 import RESISC45
from .ucmercedland import UCMercedLand
from .eurosat import EuroSat
from .l8sparcs import L8SPARCS
from .deepsat import DeepSat
from .landcover import LandCover
from .cowc import COWC
from .l7irish import L7Irish
from .sentinel2cloud import Sentinel2Cloud
from .spacenet7 import SpaceNet7
from .xview import XView
from .cloud38 import Cloud38
from .so2sat import So2Sat
from .l8biome import L8Biome


__all__ = [
    "DroneDeploy",
    "AerialCactus",
    "RESISC45",
    "UCMercedLand",
    "EuroSat",
    "L8SPARCS",
    "DeepSat",
    "LandCover",
    "COWC",
    "L7Irish",
    "Sentinel2Cloud",
    "SpaceNet7",
    "XView",
    "Cloud38",
    "So2Sat",
    "L8Biome",
]
