"""Configuration file for mapping counting and detection tasks."""

file_mapping_counting = {
    "Columbus_CSUAV_AFRL": "COWC_Counting_Columbus_CSUAV_AFRL.tbz",
    "Potsdam_ISPRS": "COWC_Counting_Potsdam_ISPRS.tbz",
    "Selwyn_LINZ": "COWC_Counting_Selwyn_LINZ.tbz",
    "Toronto_ISPRS": "COWC_Counting_Toronto_ISPRS.tbz",
    "Utah_AGRC": "COWC_Counting_Utah_AGRC.tbz",
    "Vaihingen_ISPRS": "COWC_Counting_Vaihingen_ISPRS.tbz",
}

file_mapping_detection = {
    "Columbus_CSUAV_AFRL": "COWC_Detection_Columbus_CSUAV_AFRL.tbz",
    "Potsdam_ISPRS": "COWC_Detection_Potsdam_ISPRS.tbz",
    "Selwyn_LINZ": "COWC_Detection_Selwyn_LINZ.tbz",
    "Toronto_ISPRS": "COWC_Detection_Toronto_ISPRS.tbz",
    "Utah_AGRC": "COWC_Detection_Utah_AGRC.tbz",
    "Vaihingen_ISPRS": "COWC_Detection_Vaihingen_ISPRS.tbz",
}
